﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace Bonfire
{
    public enum State
    {
        Died,
        Walking,
        AttackR1,
        AttackR1_2,
        Rolling,
        Jump
    };

    public class PlayerCtrl : MonoBehaviour
    {
        [SerializeField] Text debugText = null;

        // 入力関連
        [SerializeField] float joystickDead = 0.3f;
        float inputHorizontalStickL = 0f;
        float inputVerticalStickL = 0f;

        // 状態管理
        State state = State.Walking;        // 現在の状態
        State nextState = State.Walking;    // 次の状態
        bool isStateEnd = true;

        // 走っている、全力疾走しているフラグ
        public bool isRunning = false;
        public bool isSprint = false;

        // xボタンの入力が、ローリングからダッシュになるまでの時間を計算するための変数
        float timeSinceInputFire1 = 0f;

        PlayerStatus status = null;
        PlayerMove playerMove = null;
        HudController hudController = null;


        void Start()
        {
            status = GetComponent<PlayerStatus>();
            playerMove = GetComponent<PlayerMove>();
            hudController = FindObjectOfType<HudController>();
        }

        void Update()
        {
            UpdateTimer();
            inputHorizontalStickL = Input.GetAxis("Horizontal Stick-L");
            inputVerticalStickL = Input.GetAxis("Vertical Stick-L") * -1f; // 上下が逆なので -1 をかけておく

            // 呼び出しが多いので、短い文字数で呼べるようにしておく
            isStateEnd = status.isAnimationEnd;

            // ボタンの処理は、ここに足していく
            if (Input.GetButtonDown("Fire_Share"))
            {
                hudController.toggleHudLeft();
            }

            // 加速処理
            if (Input.GetButton("Fire_1"))
            {
                isRunning = true;
            }
            if (Input.GetButtonUp("Fire_1"))
            {
                isRunning = false;
            }

            // 全力疾走処理
            if (Input.GetButtonDown("Fire_L3"))
            {
                isSprint = isSprint ? false : true;
            }

            // 次の状態に遷移できるようになった時の処理
            if (isStateEnd)
            {
                State prevState = state;
                // StartXX 内で nextState が書き換えられるので、ここで state を更新しておく必要がある
                state = nextState;
                switch (nextState)
                {
                    case State.Walking:
                        StartWalking(prevState);
                        break;
                    case State.AttackR1:
                        StartAttackR1(prevState);
                        break;
                    case State.AttackR1_2:
                        StartAttackR1_2(prevState);
                        break;
                    case State.Rolling:
                        StartRolling(prevState);
                        break;
                    case State.Jump:
                        StartJump(prevState);
                        break;
                }
            }

            if (debugText != null)
            {
                debugText.text =
                   string.Format("state: {0}\nnexState: {1}\nisStateEnd: {2}\nisGrounded: {3}",
                                state,
                                nextState,
                                isStateEnd,
                                status.isGrounded
                               );
            }

        }

        //
        // コントローラーの入力を返すためのメソッド
        //
        public Vector3 GetPlayerDirection()
        {
            return Vector3.Normalize(new Vector3(inputHorizontalStickL, 0, inputVerticalStickL));
        }

        public float GetMagunitudeStickL()
        {
            float ret = new Vector3(inputHorizontalStickL, 0, inputVerticalStickL).magnitude;
            return ret;
        }

        public bool IsInputStickL()
        {
            float h = inputHorizontalStickL;
            float v = inputVerticalStickL;
            bool ret = ((h > joystickDead) || (h < -joystickDead) || (v > joystickDead) || (v < -joystickDead));
            return ret;
        }

        //
        // 状態(State)を開始するときに行う処理 
        // 
        private void StartWalking(State prevState)
        {
            StateStartCommon();

            // Walking のときだけは isAnimationEnd = true で扱う
            status.isAnimationEnd = true;
        }

        private void StartAttackR1(State prevState)
        {
            StateStartCommon();
            status.isAttackR1 = true;
        }

        private void StartAttackR1_2(State prevState)
        {
            StateStartCommon();
            status.isAttackR1_2 = true;
        }

        private void StartRolling(State prevState)
        {
            StateStartCommon();
            status.isRolling = true;
        }

        private void StartJump(State prevState)
        {
            StateStartCommon();
            status.isJumping = true;

            playerMove.AddJumpPower();
        }

        //
        // 共通処理のメソッド
        //
        private void StateStartCommon()
        {
            //Debug.Log("StateStartCommon()");
            status.resetStatus();
            status.isAnimationEnd = false;

            nextState = State.Walking;
        }

        //
        // その他のメソッド
        //
        private void UpdateTimer()
        {
            timeSinceInputFire1 += Time.deltaTime;
        }
    }

}
